<?php

$config = [
	'host' => 'localhost',
	'dbname' => 'kasyanov_chat',
	'username' => 'root',
	'password' => '',
];

require 'idiorm.php';
ORM::configure('mysql:host=' . $config['host'] . ';dbname=' . $config['dbname'] . '');
ORM::configure('username', $config['username']);
ORM::configure('password', $config['password']);
ORM::configure('driver_options', [PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES utf8']);

class App
{
	private static function filterUsernameAndChatName($username)
	{
		$username = htmlentities($username);
		$username = mb_substr($username, 0, 255);
		return $username;
	}
	
	private static function filterMessageText($messageText)
	{
		return htmlentities($messageText);
	}

	
	/**
	 * Создание нового чата
	 * @param string $chatName - название чата
	 * @param string $username - имя пользователя
	 */
	public static function createChatByUsername($chatName, $username)
	{
		
		$username = self::filterUsernameAndChatName($username);
		$chatName = self::filterUsernameAndChatName($chatName);
		
		if(empty($username) || empty($chatName)) {
			return false;
		}
		
		$chat = ORM::for_table('chats')->create();
		$chat->name = self::filterUsernameAndChatName($chatName);
		$chat->save();
		$chatId = $chat->id();
		
		$user = ORM::for_table('users')->create();
		$user->chat_id = $chatId;
		$user->username = $username;
	
		$user->last_activity = time();
		$user->save();
		$userId = $user->id();
		
		return [
			'chatId' => $chatId,
			'userId' => $userId,
		];
	}
	
	/**
	 * Присоединение к существующему чату.
	 */
	public static function joinToChatByUsername($chatId, $username)
	{
		$username = self::filterUsernameAndChatName($username);
		if(empty($username)) {
			return false;
		}
		$chat = ORM::for_table('chats')->find_one($chatId);
		if ($chat) {
			$chatId = $chat->id();
			
			$user = ORM::for_table('users')->where(['chat_id' => $chatId, 'username' => $username])->find_one();
			if ($user) {
				return false;
			} else {
				$user = ORM::for_table('users')->create();
				$user->chat_id = $chatId;
				$user->username = $username;
				$user->last_activity = time();
				$user->save();
				$userId = $user->id();
				return [
					'chatId' => $chatId,
					'userId' => $userId,
				];
			}
		} else {
			return false;
		}
	}
	
	/**
	 * Проверка доступа.
	 */
	public static function checkAccess($userId, $chatId)
	{
		return (bool)ORM::for_table('users')->where([
			'id' => $userId,
			'chat_id' => $chatId,
		])->find_one();
	}
	
	/**
	 * Отметка активности пользователя.
	 */
	public static function updateActivity($userId)
	{
		$user = ORM::for_table('users')->find_one($userId);
		$user->last_activity = time();
		$user->save();
	}
	
	/**
	 * Отправка сообщения.
	 */
	public static function sendMessage($userId, $messageText)
	{
		$message = ORM::for_table('messages')->create();
		$message->user_id = $userId;
		$message->message = self::filterMessageText($messageText);
		$message->created_at = time();
		$message->save();
	}
	
	/**
	 * Получение списка сообщений и идентификтаора последнего сообщения.
	 */
	public static function getMessages($chatId, $lastMsgId = 0)
	{
		$messages =
			ORM::for_table('messages')
			->select('messages.*')
			->select('users.username')
			->inner_join('users', ['messages.user_id', '=', 'users.id'])
			->order_by_asc('messages.id')
			->where(['users.chat_id' => $chatId])
			->where_gt('messages.id', $lastMsgId)
			->find_many();
		
		$result = [];
		$lastId = $lastMsgId;
		foreach ($messages as $message) {
			$result[] = [
				'username' => $message->username,
				'message' => $message->message,
				'datetime' => date('d.m.Y H:i:s', $message->created_at),
			];
			$lastId = $message->id;
		}
		return [
			'messages' => $result,
			'lastId' => $lastId
		];
	}
	
	/**
	 * Получение кол-ва пользователей онлайн.
	 */
	public static function getOnlineUsersCount($chatId)
	{
		$count = ORM::for_table('users')
			->where(['chat_id' => $chatId])
			->where_gte('last_activity', time() - 30)
			->count();
		return $count;
	}
	
	/**
	 * Получение информации о чате.
	 */
	public static function getChatInfo($chatId)
	{
		$chat = ORM::for_table('chats')->find_one($chatId);
		if ($chat) {
			return [
				'name' => $chat->name,
			];
		} else {
			return false;
		}
	}
}
